#!/usr/bin/env python2
#
# Copyright 3D Control Systems, Inc. All Rights Reserved 2017-2019. Built in San Francisco.
#
# This software is distributed under commercial non-GPL license for personal, educational,
# corporate or any other use. The software as a whole or any parts of that are prohibited
# for distribution and/or use without obtaining license from 3D Control Systems, Inc.
#
# If you do not have the license to use this software, please delete all software files
# immediately and contact sales to obtain the license: sales@3dprinteros.com.
# If you are unsure about the licensing please contact directly our sales: sales@3dprinteros.com.

import gi
gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')
gi.require_version('Notify', '0.7')
from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import Notify

import tray_common


def show_notification(title, body):
    try:
        Notify.Notification.new(title, body).show()
    except gi.repository.GLib.Error:
        pass

def show_message(data=None):
    msg = Gtk.MessageDialog(None, Gtk.DialogFlags.MODAL, Gtk.MessageType.INFO, Gtk.ButtonsType.OK, data)
    msg.run()
    msg.destroy()

def open_browser_tab(data):
    tray_common.open_browser_tab()

def quit_response(widget, response_id):
    if response_id == Gtk.ResponseType.YES:
        Gtk.main_quit()
        stop_3dprinteros()

def quit(data):
    message = "Are sure you want to quit " + tray_common.APP_NAME + "?"
    msg = Gtk.MessageDialog(None, 0,
            Gtk.MessageType.QUESTION, Gtk.ButtonsType.YES_NO, message)
    msg.connect("response", quit_response)
    msg.run()
    msg.destroy()

def stop_3dprinteros():
    show_notification('Closing ' + tray_common.APP_NAME, ' ')
    tray_common.stop_app()

def make_menu(event_button, event_time, data=None):
    menu = Gtk.Menu()
    menu.set_title(tray_common.APP_NAME)
    open_tab_item = Gtk.MenuItem("Open browser tab")
    if STATUS_CHECKER.notifications:
        notifications_item = Gtk.MenuItem("Disable notifications")
        notifications_func = disable_notifications
    else:
        notifications_item = Gtk.MenuItem("Enable notifications")
        notifications_func = enable_notifications
    quit_item = Gtk.MenuItem("Quit")
    menu.append(open_tab_item)
    menu.append(notifications_item)
    menu.append(quit_item)
    open_tab_item.connect_object("activate",  open_browser_tab, None)
    notifications_item.connect_object("activate", notifications_func, None)
    quit_item.connect_object("activate", quit, None)
    open_tab_item.show()
    notifications_item.show()
    quit_item.show()
    menu.popup(None, None, None, None, event_button, event_time)

def disable_notifications(event):
    STATUS_CHECKER.toggle_notifications(False)
    show_notification('Notifications disabled',
                                'You can get status by left click on ' + tray_common.APP_NAME + ' icon')

def enable_notifications(event):
    STATUS_CHECKER.toggle_notifications(True)
    show_notification('Notifications enable',
                                'Now you will be notified about status changes in real time')

def on_right_click(data, event_button, event_time):
    make_menu(event_button, event_time)

def on_left_click(_):
    STATUS_CHECKER.show_status_balloons()


if __name__ == '__main__':
    Notify.init(tray_common.APP_NAME)
    STATUS_CHECKER = tray_common.StatusChecker(show_notification)
    icon = Gtk.StatusIcon()
    icon.set_from_file(tray_common.ICON_PATH)
    icon.connect('popup-menu', on_right_click)
    icon.connect('activate', on_left_click)
    Gdk.threads_init()
    Gdk.threads_enter()
    try:
        Gtk.main()
    except KeyboardInterrupt:
        print('KeyboardInterrupt received, closing Gtk_tray')
    STATUS_CHECKER.stop()
    Gdk.threads_leave()
