// Copyright 3D Control Systems, Inc. All Rights Reserved 2017-2019.
// Built in San Francisco.

// This software is distributed under a commercial license for personal,
// educational, corporate or any other use.
// The software as a whole or any parts of it is prohibited for distribution or
// use without obtaining a license from 3D Control Systems, Inc.

// All software licenses are subject to the 3DPrinterOS terms of use
// (available at https://www.3dprinteros.com/terms-and-conditions/),
// and privacy policy (available at https://www.3dprinteros.com/privacy-policy/)

import QtQuick 2.0
import AppControls 1.0

import AppDialogs 1.0
import AppStyle 1.0
import "constants.js" as Constants

Item {
    id: settingsView

    ConfirmDialog {
        id: confirmOfflineMode
        visible: false
        z: 50
        anchors {
            verticalCenterOffset: -AppStyle.headHeight * 0.5
            centerIn: parent
        }
        yesText: coreModel.offlineMode ? qsTr("Reconnect") : qsTr("Disconnect")
        noText: qsTr("Cancel")
        msg: coreModel.offlineMode ? qsTr("Are you sure that you want to reconnect to the Cloud?")
                                   : qsTr("Are you sure that you want to disconnect from the Cloud?")
        onYes: {
            //TODO: refactor and move this logic to model

            if (coreModel.offlineMode && !printerModel.gotAuthToken) {
                printerModel.disconnect_printer()
            }

            coreModel.toggleOfflineMode()

            if (printerModel.gotAuthToken) {
                rootItem.hideSetting()
                return
            }

            rootItem.hideSettingWithBusy()
        }
    }

    Grid {
        id: settingsGrid
        anchors {
            centerIn: parent
            verticalCenterOffset: 15
        }
        rows: 3
        columns: 3
        columnSpacing: 30
        rowSpacing: 30

        TextButton {
            width: 200
            height: 160
            text: qsTr("Network")
            onClicked: {
                rootItem.pushView("SelectWifiView.qml")
            }
        }

        TextButton {
            width: 200
            height: 160
            text: qsTr("Advanced")
            onClicked: {
                rootItem.pushView("AdvancedView.qml")
            }
        }

        TextButton {
            width: 200
            height: 160
            text: qsTr("About")
            onClicked: {
                rootItem.pushView("AboutView.qml")
            }
        }

        TextButton {
            width: 200
            height: 160
            text: qsTr("Joystick")
            onClicked: {
                if (coreModel.state === Constants.States.PRINTING || coreModel.state === Constants.States.PAUSED) {
                    rootItem.showMessageDialog(Constants.MessageTypes.INFO, qsTr("Can't enter joystick while printing"), "OK")
                    return
                }

                if (coreModel.state === Constants.States.REGISTRATION) {
                    rootItem.showMessageDialog(Constants.MessageTypes.INFO, qsTr("Can't enter joystck while in registeration mode"), "OK")
                    return
                }

                rootItem.pushView("JoystickView.qml")
            }
        }

        TextButton {
            width: 200
            height: 160
            text: coreModel.offlineMode ? qsTr("Leave offline mode") : qsTr("Enter offline mode")
            onClicked: {
                if (coreModel.offlineMode && (coreModel.state === Constants.States.PRINTING || coreModel.state === Constants.States.PAUSED)) {
                    rootItem.showMessageDialog(Constants.MessageTypes.INFO, qsTr("Please cancel the print before leave offline mode"), "OK")
                    return
                }
                confirmOfflineMode.visible = true
            }
        }

        TextButton {
            width: 200
            height: 160
            backgroundColor: AppStyle.foreground
            textPressedColor: AppStyle.textColor
            textColor: "white"
            pressedColor: "transparent"
            borderPressedColor: AppStyle.foreground
            text: qsTr("Back")
            onClicked: {
                rootItem.hideSetting()
            }
        }
    }
}

